# Copyright (C) 2004,2005 by SICEm S.L. and Imendio AB
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk

from gazpacho.commandmanager import command_manager
from gazpacho.command import Command
from gazpacho.i18n import _
from gazpacho.properties import prop_registry, CustomProperty, \
     PropertyCustomEditor, TransparentProperty, StringType, ObjectType
from gazpacho.stockicons import StockIconList
from gazpacho.gadget import Gadget, load_gadget_from_widget
from gazpacho.widgets.base.base import ContainerAdaptor

def get_button_state(button):
    """Get the state of the button in the form of a tuple with the following
    fields:
      - stock_id: string with the stock_id or None if the button is not
        using a stock_id
      - notext: boolean that says if the button has only a stock icon or
        if it also has the stock label
      - label: string with the contents of the button text or None
      - image_path: string with the path of a custom file for the
        image or None
      - position: one of gtk.POS_* that specifies the position of the
        image with respect to the label
    """
    stock_id = label = image_path = position = None
    notext = False
    icon_size = gtk.ICON_SIZE_BUTTON

    use_stock = button.get_use_stock()
    child_name = None
    child = button.get_child()
    image_file_name = None
    if child:
        image_file_name = child.get_data('image-file-name')
        child_name = child.get_name()

    # it is a stock button
    if use_stock and not image_file_name:
        stock_id = button.get_label()

    # it only has a text label
    elif isinstance(child, gtk.Label):
        label = child.get_text()

    # it has an image without text. it can be stock icon or custom image
    elif isinstance(child, gtk.Image):
        if image_file_name:

            image_path = image_file_name
        else:
            stock_id = child.get_property('stock')
            if not stock_id:
                print('Unknown button image state, no stock, no filename')
        notext = True
        icon_size = child.get_property('icon-size')
    # it has custom image and text
    elif isinstance(child, gtk.Alignment):
        box = child.get_child()

        children = box.get_children()
        image_child = None
        text_child = None
        text_last = True
        for c in children:
            if isinstance(c, gtk.Image):
                image_child = c
                text_last = False
            elif isinstance(c, gtk.Label):
                text_child = c
                text_last = True

        if isinstance(box, gtk.HBox):
            if text_last:
                position = gtk.POS_LEFT
            else:
                position = gtk.POS_RIGHT
        else:
            if text_last:
                position = gtk.POS_TOP
            else:
                position = gtk.POS_BOTTOM

        if image_child:
            image_path = image_child.get_data('image-file-name')

        if text_child:
            label = text_child.get_text()
        else:
            notext = True

    return (stock_id, notext, label, image_path, position, icon_size,
            child_name)

# Command for the contents property:
class CommandSetButtonContents(Command):
    def __init__(self, gadget, stock_id=None, notext=False, label=None,
                 image_path=None, position=-1, icon_size=gtk.ICON_SIZE_BUTTON,
                 child_name=None):
        description = _('Setting button %s contents') % gadget.name
        Command.__init__(self, description)

        self.gadget = gadget
        self.stock_id = stock_id
        self.notext = notext
        self.label = label
        self.image_path = image_path
        self.position = position
        self.icon_size = icon_size
        self.child_name = child_name

    def execute(self):
        widget = self.gadget
        button = self.gadget.widget
        state = get_button_state(button)
        use_stock = widget.get_prop('use-stock')
        label = widget.get_prop('label')
        self._clear_button(button)

        if self.stock_id:
            # stock button

            if self.notext:
                image = gtk.Image()
                image.set_from_stock(self.stock_id, self.icon_size)
                image.show()
                button.add(image)
            else:
                use_stock.set(True)
                label.set(self.stock_id)

        else:
            # custom button. 3 cases:
            # 1) only text, 2) only image or 3) image and text
            if self.label and not self.image_path:
                # only text
                label.set(self.label)
            elif not self.label and self.image_path:
                # only image
                image = gtk.Image()
                image.set_from_file(self.image_path)
                image.set_data('image-file-name', self.image_path)
                image.show()
                button.add(image)
            elif self.label and self.image_path:
                # image and text
                align = gtk.Alignment(0.5, 0.5, 1.0, 1.0)
                if self.position in (gtk.POS_LEFT, gtk.POS_RIGHT):
                    box = gtk.HBox()
                else:
                    box = gtk.VBox()
                align.add(box)
                image = gtk.Image()
                image.set_from_file(self.image_path)
                image.set_data('image-file-name', self.image_path)
                label = gtk.Label(self.label)
                if '_' in self.label:
                    label.set_use_underline(True)

                if self.position in (gtk.POS_LEFT, gtk.POS_TOP):
                    box.pack_start(image)
                    box.pack_start(label)
                else:
                    box.pack_start(label)
                    box.pack_start(image)

                align.show_all()
                button.add(align)

        if button.child:
            project = self.gadget.project
            button.child.set_name('')
            load_gadget_from_widget(button, project)
            if not button.child.get_name():
                project.set_new_widget_name(button.child)
            project.add_hidden_widget(button.child)
            self.gadget.setup_widget(button)

        # save the state for undoing purposes
        (self.stock_id, self.notext, self.label,
         self.image_path, self.position, self.icon_size,
         self.child_name) = state

    def _clear_button(self, button):
        "Clear the button and set default values for its properties"
        button.set_use_stock(False)
        button.set_use_underline(False)
        button.set_label('')

        child = button.get_child()
        if child:
            button.remove(child)
            project = self.gadget.project
            project.remove_hidden_widget(child)

# Adaptors for Button
class ButtonContentsEditor(PropertyCustomEditor):

    wide_editor = True

    def __init__(self):
        self.loading = False
        self.gadget = None
        self.application_window = None
        self.input = self.create()

    def get_editor_widget(self):
        return self.input

    def _create_check_entry_button(self, table, row, check_text, button_text,
                                   entry_handler, check_handler,
                                   button_handler):
        check = gtk.CheckButton(check_text)
        table.attach(check, 0, 1, row, row + 1,
                     xoptions=gtk.FILL, yoptions=gtk.FILL)
        hbox = gtk.HBox()
        entry = gtk.Entry()
        entry.connect('changed', entry_handler)
        hbox.pack_start(entry)
        button = gtk.Button(button_text)
        button.connect('clicked', button_handler)
        hbox.pack_start(button, False, False)
        table.attach(hbox, 1, 2, row, row + 1, xoptions=gtk.EXPAND|gtk.FILL,
                     yoptions=gtk.EXPAND|gtk.FILL)
        check.connect('toggled', check_handler)
        return check, entry, button

    def _create_label(self, table):
        "Create the widgets needed to set the label of a button"
        c, e, b = self._create_check_entry_button(table, 0, _('Label:'), '...',
                                                  self._on_entry_label_changed,
                                                  self._on_check_label_toggled,
                                                  self._on_button_label_clicked)
        self.check_label, self.entry_label, self.button_label = c, e, b


    def _create_image(self, table):
        "Create the widgets needed to set the image of a button"
        c, e, b = self._create_check_entry_button(table, 1, _('Image:'), '...',
                                                  self._on_entry_image_changed,
                                                  self._on_check_image_toggled,
                                                  self._on_button_image_clicked)
        self.check_image, self.entry_image, self.button_image = c, e, b


    def _create_position(self, table):
        """Create the widgets needed to set the combo with the position
        of the image"""
        label = gtk.Label(('Image position:'))
        label.set_sensitive(False)
        table.attach(label, 0, 1, 2, 3, xoptions=gtk.FILL, yoptions=gtk.FILL)

        liststore = gtk.ListStore(str, int)
        liststore.append((_('Left'), gtk.POS_LEFT))
        liststore.append((_('Right'), gtk.POS_RIGHT))
        liststore.append((_('Top'), gtk.POS_TOP))
        liststore.append((_('Bottom'), gtk.POS_BOTTOM))
        combo = gtk.ComboBox(liststore)
        renderer = gtk.CellRendererText()
        combo.pack_start(renderer)
        combo.add_attribute(renderer, 'text', 0)

        combo.set_sensitive(False)
        table.attach(combo, 1, 2, 2, 3, xoptions=gtk.EXPAND|gtk.FILL,
                     yoptions=gtk.EXPAND|gtk.FILL)

        combo.connect('changed', self._on_combo_position_changed)
        self.combo_position = combo
        self.label_position = label

    def _create_stock_widgets(self):
        "Create the widgets needed to setup a stock button"
        alignment = gtk.Alignment(1.0, 0.0, 1.0, 1.0)
        alignment.set_padding(0, 12, 48, 0)
        vbox = gtk.VBox()

        self.stock = StockIconList()
        self.stock.connect('changed', self._on_stock_list_changed)
        self.stock.set_sensitive(True)
        vbox.pack_start(self.stock, False, False, padding=6)

        hbox = gtk.HBox()
        vbox.pack_start(hbox, False, False)

        self.check_notext = gtk.CheckButton(_('No text'))
        self.check_notext.connect('toggled', self._on_check_notext_toggled)
        hbox.pack_start(self.check_notext, False, False)

        label = gtk.Label(_('Icon size:'))
        hbox.pack_start(label, False, False, 6)

        self.icon_size = gtk.ComboBox()
        self.icon_size.set_sensitive(False)
        self.icon_size.connect('changed', self._on_icon_size_changed)
        cell = gtk.CellRendererText()
        self.icon_size.pack_start(cell, True)
        self.icon_size.add_attribute(cell, 'text', 0)
        model = gtk.ListStore(str, object)
        self.icon_size.set_model(model)
        for enum in list(gtk.IconSize.__enum_values__.values()):
            if enum == gtk.ICON_SIZE_INVALID:
                continue
            iter = model.append([enum.value_nick.capitalize(), enum])
            if enum == gtk.ICON_SIZE_BUTTON:
                self.icon_size.set_active_iter(iter)
        hbox.pack_start(self.icon_size, False, False)

        alignment.add(vbox)
        return alignment

    def _create_custom_widgets(self):
        "Create the widgets needed to setup a custom button"
        alignment = gtk.Alignment(1.0, 0.0, 1.0, 1.0)
        alignment.set_padding(0, 12, 48, 0)
        table = gtk.Table(rows=3, columns=2)
        table.set_col_spacings(6)
        table.set_row_spacings(2)

        self._create_label(table)

        self._create_image(table)

        self._create_position(table)

        alignment.add(table)

        return alignment

    def create(self):
        "Create the whole editor with all the widgets inside"

        mainvbox = gtk.VBox()

        # widgets to setup a custom button
        self.radio_custom = gtk.RadioButton(None, _('Use custom button:'))
        mainvbox.pack_start(self.radio_custom, False, False)

        self.custom_widgets = self._create_custom_widgets()
        mainvbox.pack_start(self.custom_widgets, True, True)

        # widgets to setup a stock button
        self.radio_stock = gtk.RadioButton(self.radio_custom,
                                           _('Use stock button:'))
        self.radio_stock.connect('toggled', self._on_radio_toggled)
        mainvbox.pack_start(self.radio_stock, False, False)
        self.stock_widgets = self._create_stock_widgets()
        mainvbox.pack_start(self.stock_widgets, True, True)

        # use an extra alignment to add some padding to the top
        al = gtk.Alignment(0.0, 0.0, 1.0, 1.0)
        al.set_padding(12, 0, 0, 0)
        al.add(mainvbox)
        al.show_all()
        return al

    def _on_radio_toggled(self, button):
        if self.loading:
            return

        # this is emitted when the stock radio is toggled
        value = self.radio_stock.get_active()
        self.stock_widgets.set_sensitive(value)
        self.custom_widgets.set_sensitive(not value)

        if value:
            self.set_stock()
        else:
            self.set_custom()

    def _on_check_label_toggled(self, button):
        if self.loading:
            return

        value = self.check_label.get_active()
        self.entry_label.set_sensitive(value)
        self.button_label.set_sensitive(value)
        self._set_position_sensitive(value and self.check_image.get_active())

        self.set_custom()

    def _on_check_image_toggled(self, button):
        if self.loading:
            return

        value = self.check_image.get_active()
        self.entry_image.set_sensitive(value)
        self.button_image.set_sensitive(value)
        self._set_position_sensitive(value and self.check_label.get_active())

        self.set_custom()

    def _set_position_sensitive(self, value):
        self.label_position.set_sensitive(value)
        self.combo_position.set_sensitive(value)

    def _on_stock_list_changed(self, stocklist, stock_id):
        if not self.loading:
            self.set_stock()

    def _on_check_notext_toggled(self, button):
        if not self.loading:
            self.set_stock()

        self.icon_size.set_sensitive(button.get_active())

    def _on_icon_size_changed(self, combobox):
        if not self.loading:
            self.set_stock()

    def _on_entry_label_changed(self, entry):
        if not self.loading:
            self.set_custom()

    def _on_entry_image_changed(self, entry):
        if not self.loading:
            self.set_custom()

    def _on_button_label_clicked(self, button):
        pass

    def _on_button_image_clicked(self, button):
        dialog = gtk.FileChooserDialog(_('Open image ...'),
                                       self.application_window,
                                       gtk.FILE_CHOOSER_ACTION_OPEN,
                                       (gtk.STOCK_CANCEL,
                                        gtk.RESPONSE_CANCEL,
                                         gtk.STOCK_OPEN,
                                        gtk.RESPONSE_OK))
        file_filter = gtk.FileFilter()
        file_filter.set_name("Images")
        file_filter.add_mime_type("image/png")
        file_filter.add_mime_type("image/jpeg")
        file_filter.add_mime_type("image/gif")
        file_filter.add_mime_type("image/x-xpixmap")
        file_filter.add_pattern("*.png")
        file_filter.add_pattern("*.jpg")
        file_filter.add_pattern("*.gif")
        file_filter.add_pattern("*.xpm")

        dialog.add_filter(file_filter)

        response = dialog.run()
        filename = dialog.get_filename()
        if response == gtk.RESPONSE_OK and filename:
            self.entry_image.set_text(filename)

        dialog.destroy()

    def _on_combo_position_changed(self, combo):
        if not self.loading:
            self.set_custom()

    def set_stock(self):
        if not self.gadget:
            return

        stock_id = self.stock.get_stock_id()
        notext = self.check_notext.get_active()
        model = self.icon_size.get_model()
        icon_size = model[self.icon_size.get_active_iter()][1]
        child_name = None
        if self.gadget.widget.child:
            child_name = self.gadget.widget.child.get_name()
        cmd = CommandSetButtonContents(self.gadget, stock_id=stock_id,
                                       notext=notext, icon_size=icon_size,
                                       child_name=child_name)
        command_manager.execute(cmd, self.gadget.project)

    def set_custom(self):
        if not self.gadget:
            return

        label = None
        if self.check_label.get_active():
            label = self.entry_label.get_text() or None

        image = None
        if self.check_image.get_active():
            image = self.entry_image.get_text() or None

        position = None
        ac_iter = self.combo_position.get_active_iter()
        if ac_iter:
            model = self.combo_position.get_model()
            position = model[ac_iter][1]

        child_name = None
        if self.gadget.widget.child:
            child_name = self.gadget.widget.child.get_name()

        cmd = CommandSetButtonContents(self.gadget, label=label,
                                       image_path=image, position=position,
                                       child_name=child_name)
        command_manager.execute(cmd, self.gadget.project)


    def _set_combo_active_position(self, pos):
        for row in self.combo_position.get_model():
            if row[1] == pos:
                self.combo_position.set_active_iter(row.iter)
                break

    def update(self, context, widget, proxy):
        self.gadget = Gadget.from_widget(widget)
        self.application_window = context.get_application_window()

        self.loading = True

        # default values for some widgets
        self.check_notext.set_active(False)
        self.check_label.set_active(True)
        self.entry_label.set_text("")
        self.check_image.set_active(False)
        self.entry_image.set_text("")
        self.combo_position.set_active(0)
        self.combo_position.set_sensitive(False)

        (stock_id, notext, label,
         image_path, position,
         icon_size, child_name) = get_button_state(widget)

        if stock_id:
            self.stock_widgets.set_sensitive(True)
            self.custom_widgets.set_sensitive(False)
            self.radio_stock.set_active(True)

            self.stock.set_stock_id(stock_id)
            self.check_notext.set_active(notext)
            for row in self.icon_size.get_model():
                if row[1] == icon_size:
                    self.icon_size.set_active_iter(row.iter)
                    break
            self.icon_size.set_sensitive(notext)
        else:
            self.stock_widgets.set_sensitive(False)
            self.custom_widgets.set_sensitive(True)
            self.radio_custom.set_active(True)

            self.check_label.set_active(True)
            self.entry_label.set_sensitive(True)
            self.button_label.set_sensitive(True)
            if label is not None:
                self.entry_label.set_text(label)

            if image_path is not None:
                self.check_image.set_active(True)
                self.entry_image.set_sensitive(True)
                self.entry_image.set_text(image_path)
            else:
                self.entry_image.set_sensitive(False)
                self.button_image.set_sensitive(False)

            if label and image_path:
                self.combo_position.set_sensitive(True)
                self._set_combo_active_position(position)
            else:
                self.combo_position.set_sensitive(False)

        if self.gadget.widget.child:
            self.gadget.widget.child.set_name(child_name)
        self.loading = False

class ButtonContentsAdaptor(TransparentProperty, StringType):
    """This adaptor allow the user the choose between a stock button and
    a custom button.

    A stock button is a button with a stock-icon. The text is optional
    A custom button allow the user to add a label, an image and the position
    of the image with respect to the label.
    """
    custom_editor = ButtonContentsEditor

    def get(self):
        return get_button_state(self.object)

    def save(self):
        return None

prop_registry.override_property('GtkButton::contents', ButtonContentsAdaptor)

class LabelAdaptor(CustomProperty, StringType):

    editable = False

    def is_translatable(self):
        # If a stock id is set, do not make it translatable,
        # because we don't want to translate stock-id labels!
        stock_id = get_button_state(self.object)[0]
        if stock_id:
            return False

        return True

    def save(self):
        (stock_id, notext,
         label, image_path) = get_button_state(self.object)[:4]
        if ((stock_id and notext)
            or (not label and image_path)
            or (label and image_path)):
            return
        else:
            return self.object.get_property('label')

prop_registry.override_simple('GtkButton::label', LabelAdaptor)

class ButtonAdaptor(ContainerAdaptor):
    def load(self, context, widget):
        # XXX: Copied from widget adaptor, there ought to be a
        #      better way of saying "I do not want to any load children"

        project = context.get_project()
        gadget = Gadget.load(widget, project)

        # if the widget is a toplevel we need to attach the accel groups
        # of the application
        if gadget.is_toplevel():
            gadget.setup_toplevel()

        return gadget

    def save(self, context, gadget):
        """Create Gadgets for the possible children of
        this widget so they are saved with the button"""
        gtk_button = gadget.widget

        (stock_id, notext, label,
         image_path) = get_button_state(gtk_button)[:4]

        child = gtk_button.get_child()
        project = context.get_project()

        if ((stock_id and notext)         # case 1: stock item without text
            or (not label and image_path) # case 2: only image
            or (label and image_path)):   # case 3: text and image

            # XXX: load_widget should also check so widget is not
            #      loaded twice
            if not Gadget.from_widget(child):
                name = child.get_data('gazpacho::object-id')
                if not name:
                    self._create_names_for_internal_widgets(child, project)
                else:
                    child.set_name(name)
                load_gadget_from_widget(child, project)
                project.add_hidden_widget(child)

            # case 4: stock item with text (nothing to do)

            # case 5: only text (nothing to do)

        # FIXME: Under some circumstances we end up with a GtkAlignment
        #        when we don't really need it.
        #        Find out when it's not needed and remove it

    def _create_names_for_internal_widgets(self, widget, project):
        project.set_new_widget_name(widget)

        if not isinstance(widget, gtk.Container):
            return

        for child in widget.get_children():
            self._create_names_for_internal_widgets(child, project)

    def fill_empty(self, context, widget):
        pass

    def post_create(self, context, button, interactive):
        button.set_label(button.get_name())

# GtkButton
prop_registry.override_simple('GtkButton::use-stock', editable=False)

# GtkCheckButton
prop_registry.override_simple('GtkCheckButton::draw-indicator', default=True)

# GtkRadioButton
class RadioGroupProp(CustomProperty, ObjectType):
    # We need to override get_default, since
    # the standard mechanism to get the initial value
    # is calling get_property, but ::group is not marked as readable
    editable = True

    def get_default(self, gobj):
        return

    def get(self):
        groups = self.object.get_group()
        if len(groups) <= 1:
            return None

        for group in groups:
            if group.get_active():
                return group

    def set(self, group):
        if not group in self.object.get_group():
            self.object.set_group(group)

    def save(self):
        value = self.get()
        if not value or value == self.object:
            return
        return value.get_name()

prop_registry.override_simple('GtkRadioButton::group', RadioGroupProp)



from decimal import Decimal
from django import forms
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _
from l10n.l10n_settings import get_l10n_default_currency_symbol
from livesettings import config_value
from satchmo_utils.numbers import round_decimal
import logging
from django.utils.html import escape


log = logging.getLogger('satchmo_utils.widgets')

def _render_decimal(value, places=2, min_places=2):

    if value is not None:
        roundfactor = "0." + "0"*(places-1) + "1"
        if value < 0:
            roundfactor = "-" + roundfactor
        
        value = round_decimal(val=value, places=places, roundfactor=roundfactor, normalize=True)
        log.debug('value: %s' % type(value))
        parts = ("%f" % value).split('.')
        n = parts[0]
        d = ""
    
        if len(parts) > 0:
            d = parts[1]
        elif min_places:
            d = "0" * min_places
        
        while len(d) < min_places:
            d = "%s0" % d
        
        while len(d) > min_places and d[-1] == '0':
            d = d[:-1]
    
        if len(d) > 0:
            value = "%s.%s" % (n, d)
        else:
            value = n
    return value

class BaseCurrencyWidget(forms.TextInput):
    """
    A Text Input widget that shows the currency amount
    """
    def __init__(self, attrs={}):
        final_attrs = {'class': 'vCurrencyField'}
        if attrs is not None:
            final_attrs.update(attrs)
        super(BaseCurrencyWidget, self).__init__(attrs=final_attrs)
        
class CurrencyWidget(BaseCurrencyWidget):
    
    def render(self, name, value, attrs=None):
        if value != '':
            value = _render_decimal(value, places=8)
        rendered = super(CurrencyWidget, self).render(name, value, attrs)
        curr = get_l10n_default_currency_symbol()
        curr = curr.replace("_", "&nbsp;")
        return mark_safe('<span class="currency">%s</span>%s' % (curr, rendered))

class TruncatedCurrencyWidget(BaseCurrencyWidget):
    """
    A Text Input widget that shows the currency amount - stripped to two digits by default.
    """
                
    def render(self, name, value, attrs=None):
        value = _render_decimal(value, places=2)
        rendered = super(TruncatedCurrencyWidget, self).render(name, value, attrs)
        curr = config_value('LANGUAGE','CURRENCY')
        curr = curr.replace("_", "&nbsp;")
        return mark_safe('<span class="currency">%s</span>%s' % (curr, rendered))
        
class StrippedDecimalWidget(forms.TextInput):
    """
    A textinput widget that strips out the trailing zeroes.
    """

    def __init__(self, attrs={}):
        final_attrs = {'class': 'vDecimalField'}
        if attrs is not None:
            final_attrs.update(attrs)
        super(StrippedDecimalWidget, self).__init__(attrs=final_attrs)

    def render(self, name, value, attrs=None):
        value = _render_decimal(value, places=8, min_places=0)
        return super(StrippedDecimalWidget, self).render(name, value, attrs)


class ReadOnlyWidget(forms.Widget):
    def render(self, name, value, attrs):
        final_attrs = self.build_attrs(attrs, name=name)
        if hasattr(self, 'initial'):
            value = self.initial
        if value:
            return mark_safe("<p>%s</p>" % escape(value))
        else:
            return ''

    def _has_changed(self, initial, data):
        return False




"""
Used to manage raw inventory and supplier relationships.  This is still
under heavy development.
"""

from django.db import models
from django.utils.translation import ugettext_lazy as _
from satchmo_store.contact.models import Contact, Organization
import datetime

class RawItem(models.Model):
    """
    A raw good supplied by a supplier.  For instance, it could be a plain 
    shirt that you process to make your Item
    """
    supplier = models.ForeignKey(Organization, verbose_name=_('Supplier'))
    supplier_num = models.CharField(_("Supplier ID"), max_length=50)
    description = models.CharField(_("Description"), max_length=200)
    unit_cost = models.DecimalField(_("Unit Cost"), max_digits=6, decimal_places=2)
    inventory = models.DecimalField(_("Inventory"),  max_digits=18,  decimal_places=6)
    
    def __unicode__(self):
        return self.description
    
    class Meta:
        verbose_name = _("Raw Item")
        verbose_name_plural = _("Raw Items")

class SupplierOrder(models.Model):
    """
    An order the store owner places to a supplier for a raw good.
    """
    supplier = models.ForeignKey(Organization, verbose_name=_('Supplier'))
    date_created = models.DateField(_("Date Created"))
    order_sub_total = models.DecimalField(_("Subtotal"), max_digits=6, decimal_places=2)
    order_shipping = models.DecimalField(_("Shipping"), max_digits=6, decimal_places=2)
    order_tax = models.DecimalField(_("Tax"), max_digits=6, decimal_places=2)
    order_notes = models.CharField(_("Notes"), max_length=200, blank=True)
    order_total = models.DecimalField(_("Total"), max_digits=6, decimal_places=2)
    
    def __unicode__(self):
        return unicode(self.date_created)
    
    def _status(self):
        return(self.supplierorderstatus_set.latest('date').status)
    status = property(_status)  
    
    def save(self, **kwargs):
        """Ensure we have a date_created before saving the first time."""
        if not self.pk:
            self.date_created = datetime.date.today()
        super(SupplierOrder, self).save(**kwargs)
    
    class Meta:
        verbose_name = _("Supplier Order")
        verbose_name_plural = _("Supplier Orders")
    
class SupplierOrderItem(models.Model):
    """
    Individual line items for an order
    """
    order = models.ForeignKey(SupplierOrder)
    line_item = models.ForeignKey(RawItem, verbose_name=_('Line Item'))
    line_item_quantity = models.IntegerField(_("Line Item Quantity"), )
    line_item_total = models.DecimalField(_("Line Item Total"), max_digits=6,decimal_places=2)
    
    def __unicode__(self):
        return unicode(self.line_item_total) 

SUPPLIERORDER_STATUS = (
    (_('Sent in'), _('Sent in')),
    (_('Shipped'), _('Shipped')),
    (_('Received'), _('Received')),
)

class SupplierOrderStatus(models.Model):
    """
    Status of a supplier's order.  There will be multiple statuses as it is
    placed and subsequently processed and received.
    """
    order = models.ForeignKey(SupplierOrder)
    status = models.CharField(_("Status"), max_length=20, choices=SUPPLIERORDER_STATUS, blank=True)
    notes = models.CharField(_("Notes"), max_length=100, blank=True)
    date = models.DateTimeField(_('Date'), blank=True)
    
    def __unicode__(self):
        return self.status
        
    class Meta:
        verbose_name = _("Supplier Order Status")
        verbose_name_plural = _("Supplier Order Statuses")



from livesettings import *
from django.utils.translation import ugettext_lazy as _

PAYMENT_GROUP = ConfigurationGroup('PAYMENT_PURCHASEORDER', 
    _('Purchase Order Module Settings'), 
    ordering = 100)

config_register_list(    
    BooleanValue(PAYMENT_GROUP, 
        'LIVE', 
        description=_("Accept real payments"),
        help_text=_("False if you want to be in test mode"),
        default=False),
        
    ModuleValue(PAYMENT_GROUP,
        'MODULE',
        description=_('Implementation module'),
        hidden=True,
        default = 'payment.modules.purchaseorder'),
        
    StringValue(PAYMENT_GROUP,
        'KEY',
        description=_("Module key"),
        hidden=True,
        default = 'PURCHASEORDER'),

    StringValue(PAYMENT_GROUP,
        'LABEL',
        description=_('English name for this group on the checkout screens'),
        default = 'Purchase Order',
        help_text = _('This will be passed to the translation utility')),

    StringValue(PAYMENT_GROUP,
        'URL_BASE',
        description=_('The url base used for constructing urlpatterns which will use this module'),
        default = '^po/'),
        
    BooleanValue(PAYMENT_GROUP, 
        'EXTRA_LOGGING', 
        description=_("Verbose logging?"),
        default=False),
        
    BooleanValue(PAYMENT_GROUP, 
        'SSL', 
        description=_("Use SSL for the checkout pages?"), 
        default=False),
)



import os
import fnmatch
import shutil
import urlparse
try:
    import Image
except ImportError:
    from PIL import Image
from django.conf import settings
from django.core.cache import get_cache
from django.db.models.fields.files import ImageField
from satchmo_utils.thumbnail.text import URLify
from livesettings import config_value
import logging
log = logging.getLogger('satchmo_utils.thumbnail')

image_cache = get_cache('locmem:///')

_FILE_CACHE_TIMEOUT = 60 * 60 * 60 * 24 * 31 # 1 month
_THUMBNAIL_GLOB = '%s_t*%s'

def _get_thumbnail_path(path, width=None, height=None):
    """ create thumbnail path from path and required width and/or height.

        thumbnail file name is constructed like this:
            <basename>_t_[w<width>][_h<height>].<extension>
    """

    # one of width/height is required
    assert (width is not None) or (height is not None)

    basedir = os.path.dirname(path) + '/'
    base, ext = os.path.splitext(os.path.basename(path))

    # make thumbnail filename
    th_name = base + '_t'
    if (width is not None) and (height is not None):
        th_name += '_w%d_h%d' % (width, height)
    elif width is not None:
        th_name += '%d' % width # for compatibility with admin
    elif height is not None:
        th_name += '_h%d' % height
    th_name += ext

    return urlparse.urljoin(basedir, th_name)

def _get_path_from_url(url, root=settings.MEDIA_ROOT, url_root=settings.MEDIA_URL):
    """ make filesystem path from url """

#    if url.startswith('/'):
#        return url

    if url.startswith(url_root):
        url = url[len(url_root):] # strip media root url

    return os.path.normpath(os.path.join(root, url))

def _get_url_from_path(path, root=settings.MEDIA_ROOT, url_root=settings.MEDIA_URL):
    """ make url from filesystem path """

    if path.startswith(root):
        path = path[len(root):] # strip media root

    return urlparse.urljoin(root, path.replace('\\', '/'))

def _has_thumbnail(photo_url, width=None, height=None, root=settings.MEDIA_ROOT, url_root=settings.MEDIA_URL):
    # one of width/height is required
    assert (width is not None) or (height is not None)

    return os.path.isfile(_get_path_from_url(_get_thumbnail_path(photo_url, width, height), root, url_root))

def make_thumbnail(photo_url, width=None, height=None, root=settings.MEDIA_ROOT, url_root=settings.MEDIA_URL):
    """ create thumbnail """

    # one of width/height is required
    assert (width is not None) or (height is not None)

    if not photo_url: return None

    th_url = _get_thumbnail_path(photo_url, width, height)
    th_path = _get_path_from_url(th_url, root, url_root)
    photo_path = _get_path_from_url(photo_url, root, url_root)

    if _has_thumbnail(photo_url, width, height, root, url_root):
        # thumbnail already exists
        if not (os.path.getmtime(photo_path) > os.path.getmtime(th_path)):
            # if photo mtime is newer than thumbnail recreate thumbnail
            return th_url

    # make thumbnail

    # get original image size
    orig_w, orig_h = get_image_size(photo_url, root, url_root)
    if (orig_w is None) and (orig_h) is None:
        # something is wrong with image
        return photo_url

    # make proper size
    if (width is not None) and (height is not None):
        if (orig_w == width) and (orig_h == height):
            # same dimensions
            return None
        size = (width, height)
    elif width is not None:
        if orig_w == width:
            # same dimensions
            return None
        size = (width, orig_h)
    elif height is not None:
        if orig_h == height:
            # same dimensions
            return None
        size = (orig_w, height)

    try:
        img = Image.open(photo_path).copy()
        img.thumbnail(size, Image.ANTIALIAS)
        img.save(th_path, quality=config_value('THUMBNAIL', 'IMAGE_QUALITY'))
    except Exception, err:
        # this goes to webserver error log
        import sys
        print >>sys.stderr, '[MAKE THUMBNAIL] error %s for file %r' % (err, photo_url)
        return photo_url

    return th_url

def _remove_thumbnails(photo_url, root=settings.MEDIA_ROOT, url_root=settings.MEDIA_URL):
    if not photo_url: return # empty url

    file_name = _get_path_from_url(photo_url, root, url_root)
    import fnmatch, os
    base, ext = os.path.splitext(os.path.basename(file_name))
    basedir = os.path.dirname(file_name)
    for file in fnmatch.filter(os.listdir(basedir), _THUMBNAIL_GLOB % (base, ext)):
        path = os.path.join(basedir, file)
        try:
            os.remove(path)
        except OSError:
            # no reason to crash due to bad paths.
            log.warn("Could not delete image thumbnail: %s", path)
        image_cache.delete(path) # delete from cache

def remove_model_thumbnails(model):
    """ remove all thumbnails for all ImageFields (and subclasses) in the model """

    for obj in model._meta.fields:
        if isinstance(obj, ImageField):
            field_value = getattr(model, obj.name)
            if field_value:
                url = field_value.path
                _remove_thumbnails(url)

def make_admin_thumbnail(url):
    """ make thumbnails for admin interface """
    return make_thumbnail(url, width=120)

def make_admin_thumbnails(model):
    """ create thumbnails for admin interface for all ImageFields (and subclasses) in the model """

    for obj in model._meta.fields:
        if isinstance(obj, ImageField):
            url = getattr(model, obj.name).path
            make_thumbnail(url, width=120)

def _get_thumbnail_url(photo_url, width=None, height=None, root=settings.MEDIA_ROOT, url_root=settings.MEDIA_URL):
    """ return thumbnail URL for requested photo_url and required width and/or height

        if thumbnail file do not exists returns original URL
    """

    # one of width/height is required
    assert (width is not None) or (height is not None)

    if _has_thumbnail(photo_url, width, height, root, url_root):
        return _get_thumbnail_path(photo_url, width, height)
    else:
        return photo_url

def _set_cached_file(path, value):
    """ Store file dependent data in cache.
        Timeout is set to _FILE_CACHE_TIMEOUT (1month).
    """

    mtime = os.path.getmtime(path)
    image_cache.set(path, (mtime, value,), _FILE_CACHE_TIMEOUT)

def _get_cached_file(path, default=None):
    """ Get file content from cache.
        If modification time differ return None and delete
        data from cache.
    """

    cached = image_cache.get(path, default)
    if cached is None:
        return None
    mtime, value = cached

    if (not os.path.isfile(path)) or (os.path.getmtime(path) != mtime): # file is changed or deleted
        image_cache.delete(path) # delete from cache
        # remove thumbnails if exists
        base, ext = os.path.splitext(os.path.basename(path))
        basedir = os.path.dirname(path)
        for file in fnmatch.filter(os.listdir(basedir), _THUMBNAIL_GLOB % (base, ext)):
            os.remove(os.path.join(basedir, file))
        return None
    else:
        return value

def get_image_size(photo_url, root=settings.MEDIA_ROOT, url_root=settings.MEDIA_URL):
    """ returns image size.

        image sizes are cached (using separate locmem:/// cache instance)
    """

    path = _get_path_from_url(photo_url, root, url_root)

    size = _get_cached_file(path)
    if size is None:
        try:
            size = Image.open(path).size
        except Exception, err:
            # this goes to webserver error log
            import sys
            print >>sys.stderr, '[GET IMAGE SIZE] error %s for file %r' % (err, photo_url)
            return None, None

        if size is not None:
            _set_cached_file(path, size)
        else:
            return None, None

    return size


##################################################
## FILE HELPERS ##

def _rename(old_name, new_name):
    """ rename image old_name -> name """
    try:
        shutil.move(os.path.join(settings.MEDIA_ROOT, old_name), os.path.join(settings.MEDIA_ROOT, new_name))
        return new_name
    except IOError:
        return old_name

# BJK Note: I think this might be the way to approach it

# def rename_by_field(field, req_name, add_path=None):
#     """Rename the file in filefield `field`"""
#     if not (field and field.content):
#         return field

def rename_by_field(file_path, req_name, add_path=None):
    if file_path.strip() == '': return '' # no file uploaded

    old_name = os.path.basename(file_path)
    path = os.path.dirname(file_path)
    media_root = os.path.normpath(settings.MEDIA_ROOT)
    if path.startswith(media_root):
        path = path[len(media_root):]
    if path[0] == '/' or path[0] == '\\':        #windows fix
        path = path[1:]
    name, ext = os.path.splitext(old_name)
    new_name = URLify(req_name) + ext

    if (add_path is not None) and (add_path not in path): # prevent adding if already here
        dest_path = os.path.join(path, add_path)
    else:
        dest_path = path

    if not os.path.isdir(os.path.join(media_root, dest_path)):
        os.makedirs(os.path.join(media_root, dest_path))

    dest_path = os.path.join(dest_path, new_name)

    if file_path != dest_path:
        return _rename(file_path, dest_path).replace('\\', '/') # windows fix
    else:
        return file_path.replace('\\', '/') # windows fix



from django import template
from django.utils import translation
from livesettings import config_get_group, config_get
from satchmo_store.shop.models import ORDER_STATUS 

register = template.Library()

@register.filter
def payment_label(group):
    """convert a payment key into its translated text"""
    if not group.startswith('PAYMENT_'):
        group = "PAYMENT_" + group.upper()
    config = config_get_group(group)
    label = translation.ugettext(config.LABEL.value)
    return label.capitalize()

@register.inclusion_tag('payment/_order_payment_summary.html')
def order_payment_summary(order, paylink=False):
    """Output a formatted block giving attached payment details."""
   
    return {'order' : order,
        'paylink' : paylink}

@register.filter
def status_label(value):
    """convert a order status into its translated text"""
   
    for status, descr in ORDER_STATUS:
        if status == value:
            return descr
    return value



from django.db import models
from django.utils.translation import ugettext, ugettext_lazy as _
from product.models import TaxClass
from l10n.models import AdminArea, Country

class TaxRate(models.Model):
    """
    Actual percentage tax based on area and product class
    """
    taxClass = models.ForeignKey(TaxClass, verbose_name=_('Tax Class'))
    taxZone = models.ForeignKey(AdminArea, blank=True, null=True,
        verbose_name=_('Tax Zone'))
    taxCountry = models.ForeignKey(Country, blank=True, null=True,
        verbose_name=_('Tax Country'))
    percentage = models.DecimalField(_("Percentage"), max_digits=7,
        decimal_places=6, help_text=_("% tax for this area and type"))

    def _country(self):
        if self.taxZone:
            return self.taxZone.country.name
        else:
            return self.taxCountry.name
    country = property(_country)

    def _display_percentage(self):
        return "%#2.2f%%" % (100*self.percentage)
    _display_percentage.short_description = _('Percentage')
    display_percentage = property(_display_percentage)    

    def __unicode__(self):
        return u"%s - %s = %s" % (self.taxClass,
                             self.taxZone and self.taxZone or self.taxCountry,
                             self.display_percentage)

    class Meta:
        verbose_name = _("Tax Rate")
        verbose_name_plural = _("Tax Rates")
        
import config




import logging

log = logging.getLogger('wishlist.listener')

def wishlist_cart_add_listener(sender, request=None, method={}, **kwargs):
    """Listens for cart_add_view signal and checks to see if it is a wishlist add request.
    If so, returns the wishlist add view method.
    """

    if request and request.method == "POST":
        if request.POST.get('addwish', '') != '' or request.POST.get('addwish.x', '') != '':
            log.debug("Found addwish in post, returning the wishlist add view")
            from satchmo_ext.wishlist.views import wishlist_add
            method['view'] = wishlist_add
    


from decimal import Decimal
from django.core import mail
from django.core.urlresolvers import reverse as url
from django.test import TestCase
from django.test.client import Client
from django.utils.encoding import smart_str
from keyedcache import cache_delete
from l10n.models import Country
from l10n.utils import moneyfmt
from product.utils import rebuild_pricing
from satchmo_store.shop.satchmo_settings import get_satchmo_setting
from satchmo_store.shop.tests import get_step1_post_data

domain = 'http://example.com'
prefix = get_satchmo_setting('SHOP_BASE')
if prefix == '/':
    prefix = ''

class ShopTest(TestCase):
    fixtures = ['l10n-data.yaml', 'sample-store-data.yaml', 'products.yaml', 'test-config.yaml', 'initial_data.yaml']

    def setUp(self):
        # Every test needs a client
        cache_delete()
        self.client = Client()
        self.US = Country.objects.get(iso2_code__iexact = "US")
        rebuild_pricing()

    def tearDown(self):
        cache_delete()

    def test_custom_product(self):
        """
        Verify that the custom product is working as expected.
        """
        response = self.client.get(prefix+"/")
        self.assertContains(response, "Computer", count=1)
        response = self.client.get(prefix+"/product/satchmo-computer/")
        self.assertContains(response, "Memory", count=1)
        self.assertContains(response, "Case", count=1)
        self.assertContains(response, "Monogram", count=1)
        response = self.client.post(prefix+'/cart/add/', { "productname" : "satchmo-computer",
                                                      "5" : "1.5gb",
                                                      "6" : "mid",
                                                      "custom_monogram": "CBM",
                                                      "quantity" : '1'})
        self.assertRedirects(response, prefix + '/cart/',
            status_code=302, target_status_code=200)
        response = self.client.get(prefix+'/cart/')
        self.assertContains(response, '/satchmo-computer/">satchmo computer', count=1, status_code=200)
        amount = smart_str(moneyfmt(Decimal('168.00')))
        self.assertContains(response, amount, count=3)

        amount = smart_str('Monogram: CBM  ' + moneyfmt(Decimal('10.00')))
        self.assertContains(response, amount, count=1)

        amount = smart_str('Case - External Case: Mid  ' + moneyfmt(Decimal('10.00')))
        self.assertContains(response, amount, count=1)

        amount = smart_str('Memory - Internal RAM: 1.5 GB  ' + moneyfmt(Decimal('25.00')))
        self.assertContains(response, amount, count=1)

        response = self.client.post(url('satchmo_checkout-step1'), get_step1_post_data(self.US))
        self.assertRedirects(response, url('DUMMY_satchmo_checkout-step2'),
            status_code=302, target_status_code=200)
        data = {
            'credit_type': 'Visa',
            'credit_number': '4485079141095836',
            'month_expires': '1',
            'year_expires': '2012',
            'ccv': '552',
            'shipping': 'FlatRate'}
        response = self.client.post(url('DUMMY_satchmo_checkout-step2'), data)
        self.assertRedirects(response, url('DUMMY_satchmo_checkout-step3'),
            status_code=302, target_status_code=200)
        response = self.client.get(url('DUMMY_satchmo_checkout-step3'))

        amount = smart_str('satchmo computer - ' + moneyfmt(Decimal('168.00')))
        self.assertContains(response, amount, count=1, status_code=200)
        response = self.client.post(url('DUMMY_satchmo_checkout-step3'), {'process' : 'True'})
        self.assertRedirects(response, url('DUMMY_satchmo_checkout-success'),
            status_code=302, target_status_code=200)
        self.assertEqual(len(mail.outbox), 1)


if __name__ == "__main__":
    import doctest
    doctest.testmod()



# Copyright (C) 2005 by Async Open Source and Sicem S.L.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import os
import sys

import pango
import gtk

from gazpacho.app.bugreportdialog import send_bugreport
from gazpacho.i18n import _

class DebugWindow(gtk.Dialog):

    application = None

    def __init__(self, parent=None):
        gtk.Dialog.__init__(self, 'An Error Occurred', parent,
                            gtk.DIALOG_MODAL | gtk.DIALOG_NO_SEPARATOR)
        self._tb = None

        self._build_ui()
        self.send = self._create_button(gtk.STOCK_NETWORK,
                                        _('Send _Bugreport...'))
        self.send.connect('clicked', self._on_send__clicked)
        self.action_area.pack_end(self.send)

        save = self._create_button(gtk.STOCK_SAVE, _('_Save Project'))
        save.set_sensitive(self._should_save_be_sensitive())
        save.connect('clicked', self._on_save__clicked)
        self.action_area.pack_end(save)

        cont = self._create_button(gtk.STOCK_OK, _('_Continue'))
        self.add_action_widget(cont, gtk.RESPONSE_ACCEPT)

        self._pwd = os.getcwd()

    def _create_button(self, stock, text):
        b = gtk.Button()
        a = gtk.Alignment(0.5, 0.5, 1.0, 1.0)
        h = gtk.HBox()
        a.add(h)
        i = gtk.Image()
        i.set_from_stock(stock, gtk.ICON_SIZE_BUTTON)
        l = gtk.Label(text)
        if '_' in text:
            l.set_use_underline(True)

        h.pack_start(i)
        h.pack_start(l)
        b.add(a)
        b.show_all()
        return b

    def _build_ui(self):
        # 416 = 600 / sqrt(2) = golden number
        self.set_default_size(600, 416)
        self.set_border_width(12)

        hbox = gtk.HBox()
        self.vbox.pack_start(hbox, False, False)
        self.vbox.set_spacing(6)
        lbl = gtk.Label('<b>Exception type:</b>')
        lbl.set_use_markup(True)
        hbox.pack_start(lbl, False, False, 6)
        lbl.show()
        self._info_label = gtk.Label()
        hbox.pack_start(self._info_label, False, False)
        self._info_label.show()
        hbox.show()

        sw = gtk.ScrolledWindow()
        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        sw.set_shadow_type(gtk.SHADOW_IN)
        self.vbox.pack_start(sw)
        sw.show()

        self._buffer = gtk.TextBuffer()
        self._buffer.create_tag('filename', style=pango.STYLE_ITALIC,
                                foreground='gray20')
        self._buffer.create_tag('name', foreground='#000055')
        self._buffer.create_tag('lineno', weight=pango.WEIGHT_BOLD)
        self._buffer.create_tag('exc', foreground='#880000',
                                weight=pango.WEIGHT_BOLD)

        self._textview = gtk.TextView(self._buffer)
        self._textview.set_editable(False)
        self._textview.set_cursor_visible(False)
        sw.add(self._textview)
        self._textview.show()

    def show_exception(self, exctype, value, tb):
        import traceback
        firsttb = tb
        while firsttb.tb_next:
            firsttb = firsttb.tb_next
        self._tb = firsttb
        self._info_label.set_text(str(exctype))
        self.print_tb(tb)
        msg = traceback.format_exception_only(exctype, value)[0]
        result = msg.split(' ', 1)
        if len(result) == 1:
            msg = result[0]
            arguments = ''
        else:
            msg, arguments = result
        self._insert(msg, 'exc')
        self._insert(' ' + arguments)

    def _print(self, line):
        self._buffer.insert_at_cursor(line + '\n')

    def _insert(self, text, *tags):
        end_iter = self._buffer.get_end_iter()
        self._buffer.insert_with_tags_by_name(end_iter, text, *tags)

    def _print_file(self, filename, lineno, name):
        if filename.startswith(self._pwd):
            filename = filename.replace(self._pwd, '')[1:]

        self._insert('  File ')
        self._insert(filename, 'filename')
        self._insert(', line ')
        self._insert(str(lineno), 'lineno')
        self._insert(', in ')
        self._insert(name, 'name')
        self._insert('\n')

    def print_tb(self, tb, limit=None):
        """Print up to 'limit' stack trace entries from the traceback 'tb'.

        If 'limit' is omitted or None, all entries are printed.  If 'file'
        is omitted or None, the output goes to sys.stderr; otherwise
        'file' should be an open file or file-like object with a write()
        method.
        """

        import linecache
        if limit is None:
            if hasattr(sys, 'tracebacklimit'):
                limit = sys.tracebacklimit
        n = 0
        while tb is not None and (limit is None or n < limit):
            f = tb.tb_frame
            lineno = tb.tb_lineno
            co = f.f_code
            filename = co.co_filename
            name = co.co_name
            self._print_file(filename, lineno, name)
            line = linecache.getline(filename, lineno)
            if line:
                self._print('    ' + line.strip())
            tb = tb.tb_next
            n = n+1

    def _start_debugger(self):
        import pdb
        pdb.pm()

    def _on_send__clicked(self, button):
        exception_text = self._buffer.get_text(*self._buffer.get_bounds())
        if send_bugreport(self._tb, exception_text, parent=self):
            # the bug was sent and we don't want the user to send it again
            self.send.set_sensitive(False)
            self.hide()

    def _on_save__clicked(self, button):
        DebugWindow.application._save_cb(None)

    def _should_save_be_sensitive(self):
        if not DebugWindow.application._project:
            return False

        if not DebugWindow.application._project._changed:
            return False

        return True

def show(exctype, value, tb):
    if exctype is not KeyboardInterrupt:

        widget = gtk.grab_get_current()
        if widget is not None:
            widget.grab_remove()

        dw = DebugWindow()
        dw.show_exception(exctype, value, tb)
        dw.run()
        dw.destroy()


if __name__ == '__main__':
    dw = DebugWindow()
    resp = dw.run()
    print(resp)
    dw.destroy()




